/**
 * @file
 * Custom JS for the JSON table field formatter.
 */

(function ($, Drupal, drupalSettings) {
  'use strict';

  /**
   * Attach behavior for JSON Fields.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.json_table = {
    attach: function (context) {
      // Initialize the Quick Edit app once per page load.
      $(context).find('.field--type-json.field--widget-json-table-widget').once('json-table-field').each(function () {
        $('.add-row').on('click', function () {
          let tbody = $(this).closest('.json-table-wrapper').find('table tbody');
          let tr = tbody.children().first().clone();
          tbody.append(tr);
          pasteFromClipBoard();
          keyupInputTable();
        });
        $('.add-col').on('click', function () {
          $(this).closest('.json-table-wrapper').find('table tbody tr').append('<td><input class="form-control form-element form-text"/></td>');
          keyupInputTable();
        });
        $('.remove-row').on('click', function () {
          $(this).closest('.json-table-wrapper').find('table tbody tr:last').remove();
          convertTable2Json($(this).closest('.json-table-wrapper').find('table tbody'));
        });
        $('.remove-col').on('click', function () {
          $(this).closest('.json-table-wrapper').find('table tbody tr td:last-child').remove();
          convertTable2Json($(this).closest('.json-table-wrapper').find('table tbody'));
        });
        pasteFromClipBoard();
        keyupInputTable();
        function getHeaderArray(that){
          var header = [];
          let $table = that.closest('table'),
            header_associative = $table.data('associative');
          if(header_associative == '1'){
            // This loop is not very efficient. It crash browser.
            $table.find('thead th').each(function (key,value) {
              header[key] = value.innerHTML;
            });
          }
          return header;
        }
        function convertTable2Json(that) {
          let $textarea = that.closest('.field--widget-json-table-widget').find('textarea.json-table');
          var table = [];
          var header = getHeaderArray(that);
          that.closest('table').find('tbody tr').each(function (row, tr) {

            let rows = $(this).find('input');
            let totalCol = rows.length;
            table[row] = new Array(totalCol);
            $.each(rows, function (col, value) {
              let associative = col;
              if(header[col] !== undefined){
                associative = header[col];
              }
              table[row][associative] = $(this).val();
            });
          });
          $textarea.val(JSON.stringify(table));
        }
        function keyupInputTable(){
          $('.json-table input').on('change keyup', function (e) {
            convertTable2Json($(this))
          });
        }
        function pasteFromClipBoard() {
          $('.json-table input.paste').on('paste', function (e) {
            var $this = $(this),
              header = getHeaderArray($this);
            $.each(e.originalEvent.clipboardData.items, function (i, v) {
              if (v.type === 'text/plain') {
                v.getAsString(function (text) {
                  var tbody = $this.closest('tbody'),
                    $textarea = $this.closest('.form-item').find('textarea'),
                    table = [[]];
                  text = text.trim('\r\n');
                  tbody.html('');
                  $.each(text.split('\r\n'), function (row, v2) {
                    let rows = v2.split('\t');
                    let totalCol = rows.length;
                    table[row] = new Array(totalCol);
                    var tr = $('<tr></tr>');
                    $.each(rows, function (col, v3) {
                      let cell = 'cell-' + row + '-' + col;
                      let td = $('<td><input class="form-control form-element form-text ' + cell + '" value="' + v3 + '"/></td>');

                      let associative = col;
                      if(header[col] !== undefined){
                        associative = header[col];
                      }
                      //table[row][col] = v3;
                      table[row][associative] = v3;
                      td.appendTo(tr);
                    });
                    tr.appendTo(tbody);
                  });
                  $textarea.text(JSON.stringify(table));
                });
              }
            });
            return false;
          });
        }
      });
    }
  };
})(jQuery, Drupal, drupalSettings);
